/*
 *   8OD - Arduino form factor i8086 based SBC
 *   Matthew Millman (tech.mattmillman.com)
 *
 *   C based initialisation for the bootrom
 *
 *   This is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <i86.h>

#include "eod_io.h"
#include "pgm.h"
#include "mid.h"
#include "flash.h"
#include "util.h"

#pragma aux     _CMain  "_*";

/* Memory map (0x70000-0x7FFFF in RAM):
 *
 * 0x0000 - 0x1FFF 8K  : _DATA (DGROUP)
 * 0x1FFF - 0x7FFF 24K : STACK
 * 0x8000 - 0xFFFF 32K : HEAP
 */

/* clib */
extern void _WCFAR __HeapInit(void _WCNEAR *start, unsigned int amount);
extern void __InitFiles(void);
/* Internal */
extern void _IoInit(void);
extern void main(void);

extern uint16_t far _heap_start;
extern uint16_t far _heap_size;

extern int spi_load_and_boot(int);
extern void boot_fail(void);

void _CMain(void)
{
    //_amblksiz = 8 * 1024;   /* set minimum memory block allocation */
	_IoInit();
    mid_init(M_CLK_DIV4);
    flash_init();

    //pgm_negotiate() can get by without most initialisation
    if (!pgm_negotiate())
        spi_load_and_boot(1);
    
    /* Setup malloc() */
    __HeapInit((void _WCNEAR *)_heap_start, _heap_size);
    /* Init stdin, stdout, stderr */
    __InitFiles();

    switch (pgm_main())
    {
    case PGM_BOOT:
        spi_load_and_boot(1);
        break;
    case PGM_RESET:
        hard_reset();
        break;
    default:
        break;
    }

    hard_reset();
}
