/*
 *   8OD - Arduino form factor i8086 based SBC
 *   Matthew Millman (tech.mattmillman.com)
 *
 *   SPI flash driver
 *
 *   This is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __FLASH_H__
#define __FLASH_H__

void flash_init(void);
void flash_wait_write(void);
void flash_read(uint32_t offset, uint16_t len, uint8_t *buf);
void flash_read_to_uart(uint32_t offset, uint32_t len, int uart_index);
int flash_write(uint32_t start, uint32_t len, uint8_t *buf);
int flash_erase(uint32_t start, uint32_t len);
int flash_is_present(void);
int flash_lock_bootarea(int lock);
int flash_get_bootarea_lock_state(void);

#define M25P80_SIZE					0x100000
#define M25P80_ERASE_SIZE			0x10000
#define M25P80_PAGE_SIZE			0x100

#define FLASH_BOOT_OFFSET           0x80000

#define FLASH_SIZE                  M25P80_SIZE
#define FLASH_ERASE_SIZE            M25P80_ERASE_SIZE

#endif /* __FLASH_H__ */
